#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "terrain.h"
#include "structures.h"
#include "personnage.h"
#include "deplacements.h"

/*!
* \file case.c
* \brief Fichier contenant le code des fonctions liees aux cases
*/

void init_case(Case * c,int x,int y, Terrain *t){
    c->coord_x=x;
    c->coord_y=y;
    c->terrain=t;
    c->occupee=faux;
}

int get_x(Case *c){
    return c->coord_x;
}

int get_y(Case *c){
    return c->coord_y;
}

  /*!
 * \fn char * getID(Carte_Jeu * c)
 * \brief La fonction retourne un pointeur vers l'ID de la Carte_Jeu.
 *
 * \param Un pointeur vers la Carte_Jeu  tester
 * \return Le pointeur vers l'ID de la Carte_Jeu.
 */
char * getID(Carte_Jeu * c){
    return c->id;
}

  /*!
 * \fn Case * getCase(Carte_Jeu * c, int x, int y)
 * \brief La fonction retourne un pointeur vers la Case de la Carte_Jeu aux coordonnes x,y.
 *
 * \param Un pointeur vers la Carte_Jeu  tester et les coordonnes x et y en entier.
 * \return un pointeur vers la Case aux coordonnes x, y.
 */
Case * getCase(Carte_Jeu * c, int x, int y){
    return &c->Tab_Cases[x][y];
}

boolean est_occupee(Case *c){
    return c->occupee;
}

boolean case_franchissable(Case *c){
    return terrain_franchissable(c->terrain);
}

void marquer_occupee(Case *c){
    c->occupee=vrai;
}

void marquer_libre(Case *c){
    c->occupee=faux;
}

Terrain* get_terrain(Case *c){
    if (c->terrain==NULL){
        return NULL;
    }
    return c->terrain;
}

  /*!
 * \fn Carte * nouvelleCarte()
 * \brief La fonction alloue l'espace en memoire pour une structure Carte et renvoye un pointeur vers la structure creee
 *
 * \return Le pointeur vers la Carte creee.
 */
Carte_Jeu * nouvelleCarte(){
    Carte_Jeu * c = malloc(sizeof(Carte_Jeu));
    return c;
}

  /*!
 * \fn void deleteCarte(Carte * c)
 * \brief libere l'espace memoire alloue a une Carte
 *
 * \param le pointeur vers la Carte a supprimer.
 */
void deleteCarte(Carte_Jeu * c){
    free(c);
}

  /*!
 * \fn int initCarte(Carte_A * sourceXML, Carte_Jeu * destination, Terrain * terrainPackage)
 * \brief La fonction initialise la carte du jeu a partir de la Carte du Package passe en parametre.
 *
 * \param Un pointeur vers la Carte_A du package qui sera la source, un pointeur vers la Carte_Jeu a initialiser, un pointeur vers le Game_Package de l'univers du jeu
 * \return 1 si l'initialisation est correcte, 0 sinon.
 */
int initCarte(Carte_A * sourceXML, Carte_Jeu * destination, Game_Package * pack){
    int i,j;
    strncpy(getID(destination), sourceXML->id, TAILLE_ID);
    for(i=0;i<LARG_MAX_CARTE;i++){
        for(j=0;j<HAUT_MAX_CARTE;j++){
            switch (sourceXML->lignes[i][j]){
                case 'P' :
                    init_case(getCase(destination,i,j),i,j,&pack->Terrains_Package[0]);
                    break;
                case 'R' :
                    init_case(getCase(destination,i,j),i,j,&pack->Terrains_Package[1]);
                    break;
                case 'F' :
                    init_case(getCase(destination,i,j),i,j,&pack->Terrains_Package[2]);
                    break;
                case 'M' :
                    init_case(getCase(destination,i,j),i,j,&pack->Terrains_Package[3]);
                    break;
                default :
                    return 0;
            }
        }
    }
    return 1;
}

  /*!
 * \fn init_carte(Carte* c,unsigned short int largeur, unsigned short int hauteur)
 * \brief La fonction initialise la position des Case contenues dans la Carte et les definit comme libres.
 *        l'attribut Terrain de la Carte est initialis a NULL, il doit etre change a partir d'une autre fonction.
 *
 * \param Un pointeur vers la Carte a initialiser, des entiers representant la largeur et la hauteur de la carte.
 * \return 0 si la Carte n'a pas ete correctement initialisee (si la taille maximum de la carte est depassee), 1 sinon.
 */
int init_carte(Carte * c){
    int i,j,n;
    n=c;
    for (i=0;i<LARG_MAX_CARTE;i++){
        for (j=0;j<HAUT_MAX_CARTE;j++){
            init_case(n,i,j,NULL);
            n=n+sizeof(Case);
        }
    }
    return 1;
}
